package com.ruoyi.asset.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.asset.domain.dto.BatchTbAssetVideoDTO;
import com.ruoyi.asset.domain.dto.TbAssetVideoDTO;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.asset.domain.TbAssetVideo;
import com.ruoyi.asset.service.ITbAssetVideoService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 设备视频Controller
 * 
 * @author 原动力
 * @date 2023-05-06
 */
@RestController
@RequestMapping("/asset/assetVideo")
public class TbAssetVideoController extends BaseController
{
    @Autowired
    private ITbAssetVideoService tbAssetVideoService;

    /**
     * 查询设备视频列表
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbAssetVideo tbAssetVideo)
    {
        startPage();
        List<TbAssetVideo> list = tbAssetVideoService.selectTbAssetVideoList(tbAssetVideo);
        return getDataTable(list);
    }

    /**
     * 查询设备视频列表
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:list')")
    @GetMapping("/list2")
    public TableDataInfo list2(TbAssetVideo tbAssetVideo)
    {
        startPage();
        List<TbAssetVideoDTO> list = tbAssetVideoService.selectTbAssetVideoList2(tbAssetVideo);
        return getDataTable(list);
    }

    /**
     * 导出设备视频列表
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:export')")
    @Log(title = "设备视频", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbAssetVideo tbAssetVideo)
    {
        List<TbAssetVideo> list = tbAssetVideoService.selectTbAssetVideoList(tbAssetVideo);
        ExcelUtil<TbAssetVideo> util = new ExcelUtil<TbAssetVideo>(TbAssetVideo.class);
        util.exportExcel(response, list, "设备视频数据");
    }

    /**
     * 获取设备视频详细信息
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tbAssetVideoService.selectTbAssetVideoById(id));
    }

    /**
     * 新增设备视频
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:add')")
    @Log(title = "设备视频", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbAssetVideo tbAssetVideo)
    {
        return toAjax(tbAssetVideoService.insertTbAssetVideo(tbAssetVideo));
    }

    /**
     * 批量新增演示视频
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:add')")
    @Log(title = "设备视频", businessType = BusinessType.INSERT)
    @PostMapping("/batch")
    public AjaxResult batchAdd(@RequestBody BatchTbAssetVideoDTO batchTbAssetVideoDTO)
    {
        return toAjax(tbAssetVideoService.batchInsertTbAssetVideo(batchTbAssetVideoDTO));
    }

    /**
     * 修改设备视频
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:edit')")
    @Log(title = "设备视频", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbAssetVideo tbAssetVideo)
    {
        return toAjax(tbAssetVideoService.updateTbAssetVideo(tbAssetVideo));
    }

    /**
     * 删除设备视频
     */
    @PreAuthorize("@ss.hasPermi('asset:assetVideo:remove')")
    @Log(title = "设备视频", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tbAssetVideoService.deleteTbAssetVideoByIds(ids));
    }
}
