package com.ruoyi.asset.controller;

import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.utils.ByteUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.file.MimeTypeUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.asset.domain.TbDemoVideo;
import com.ruoyi.asset.service.ITbDemoVideoService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 演示视频Controller
 * 
 * @author 原动力
 * @date 2023-05-06
 */
@RestController
@RequestMapping("/asset/demoVideo")
public class TbDemoVideoController extends BaseController
{
    @Autowired
    private ITbDemoVideoService tbDemoVideoService;

    /**
     * 查询演示视频列表
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbDemoVideo tbDemoVideo)
    {
        startPage();
        List<TbDemoVideo> list = tbDemoVideoService.selectTbDemoVideoList(tbDemoVideo);
        return getDataTable(list);
    }

    /**
     * 查询演示视频列表
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:list')")
    @GetMapping("/searchList")
    public AjaxResult searchList(@RequestParam("videoKey") String videoKey)
    {
        if (StrUtil.isBlank(videoKey)) {
            return error("搜索键不能为空");
        }
        TbDemoVideo tbDemoVideo = new TbDemoVideo();
        tbDemoVideo.setName(videoKey);
        List<TbDemoVideo> list = tbDemoVideoService.selectTbDemoVideoList(tbDemoVideo);
        return AjaxResult.success(list);
    }

    /**
     * 导出演示视频列表
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:export')")
    @Log(title = "演示视频", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbDemoVideo tbDemoVideo)
    {
        List<TbDemoVideo> list = tbDemoVideoService.selectTbDemoVideoList(tbDemoVideo);
        ExcelUtil<TbDemoVideo> util = new ExcelUtil<TbDemoVideo>(TbDemoVideo.class);
        util.exportExcel(response, list, "演示视频数据");
    }

    /**
     * 获取演示视频详细信息
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tbDemoVideoService.selectTbDemoVideoById(id));
    }

    /**
     * 新增演示视频
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:add')")
    @Log(title = "演示视频", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbDemoVideo tbDemoVideo)
    {
        return toAjax(tbDemoVideoService.insertTbDemoVideo(tbDemoVideo));
    }

    /**
     * 上传演示视频
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:add')")
    @Log(title = "演示视频", businessType = BusinessType.INSERT)
    @PostMapping("/upload")
    public AjaxResult upload(@RequestPart("file") MultipartFile file) throws Exception{
        String username = getLoginUser().getUsername();

        String SAVE_VIDEO_PATH = RuoYiConfig.getVideoPath();
        // 获取原文件名
        String originalFilename = file.getOriginalFilename();
        long size = file.getSize();
        // 获取文件大小
        String byteSize = ByteUtils.formatByteSize(size);
        // 获取文件类型
        String contentType = file.getContentType();
        // 上传视频并获取视频存放位置
        String fileName = FileUploadUtils.upload(SAVE_VIDEO_PATH, file, MimeTypeUtils.VIDEO_EXTENSION);

        if (StrUtil.isBlank(fileName)) {
            return error("视频上传失败");
        }

        TbDemoVideo tbDemoVideo = new TbDemoVideo(originalFilename, contentType, byteSize, fileName, 0);
        tbDemoVideo.setCreateBy(username);

        int result = tbDemoVideoService.insertTbDemoVideo(tbDemoVideo);
        if (result < 1) {
            FileUtils.deleteFile(fileName);
        }

        return toAjax(result);
    }

    @Resource
    private VideoHttpRequestHandler videoHttpRequestHandler;

    @GetMapping(value = "/player")
    public void getUploadImage(HttpServletRequest request, HttpServletResponse response, @RequestParam(value = "name", required = false) String name) throws Exception{

        if (StrUtil.isNotBlank(name)) {
            String[] split = name.split("/");
            StringBuilder res = new StringBuilder();
            for (int i = 0; i < split.length; i++) {
                if (i > 2) {
                    res.append("/");
                    res.append(split[i]);
                }
            }
            name = String.valueOf(res);
        }

        Path path = Paths.get(RuoYiConfig.getVideoPath() + name);
        if (Files.exists(path)) {
            String mimeType = Files.probeContentType(path);
            if (!StrUtil.isBlank(mimeType)) {
                response.setContentType(mimeType);
            }
            request.setAttribute(VideoHttpRequestHandler.ATTR_FILE, path);
            videoHttpRequestHandler.handleRequest(request, response);
        } else {
            response.setStatus(HttpServletResponse.SC_NOT_FOUND);
            response.setCharacterEncoding(StandardCharsets.UTF_8.toString());
        }
    }

    /**
     * 修改演示视频
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:edit')")
    @Log(title = "演示视频", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbDemoVideo tbDemoVideo)
    {
        return toAjax(tbDemoVideoService.updateTbDemoVideo(tbDemoVideo));
    }

    /**
     * 删除演示视频
     */
    @PreAuthorize("@ss.hasPermi('asset:demoVideo:remove')")
    @Log(title = "演示视频", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tbDemoVideoService.deleteTbDemoVideoByIds(ids));
    }
}
