package com.ruoyi.asset.controller;

import com.ruoyi.asset.domain.TbAsset;
import com.ruoyi.asset.domain.TbDemoVideo;
import com.ruoyi.asset.domain.dto.VisitorAssetInfoDTO;
import com.ruoyi.asset.service.ITbAssetService;
import com.ruoyi.asset.service.ITbAssetVideoService;
import com.ruoyi.asset.service.ITbDemoVideoService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.system.domain.SysNotice;
import com.ruoyi.system.service.ISysNoticeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 学生访问Controller
 *
 * @author 原动力
 * @date 2023-03-27
 */
@Api(tags = "资产信息")
@RestController
@RequestMapping("/visitor")
public class VisitorController extends BaseController {
    @Resource
    private ITbAssetService assetService;

    @Resource
    private ITbAssetVideoService assetVideoService;

    @Resource
    private ITbDemoVideoService demoVideoService;

    @Resource
    private ISysNoticeService noticeService;

    /**
     * 查看公开设备列表
     */
    @ApiOperation("查看公开设备列表")
    @GetMapping("/asset/list")
    public AjaxResult assetList()
    {
        List<TbAsset> list = assetService.selectVisitorTbAsset();
        return AjaxResult.success(list);
    }

    @ApiOperation("查看公开设备详情")
    @GetMapping("/asset/detail/{barCode}")
    public AjaxResult assetDetail(@PathVariable("barCode") String assetBarCode) {
        TbAsset tbAsset = assetService.selectTbAssetByBarCode(assetBarCode);
        if (tbAsset == null) {
            return error("设备不存在");
        }
        VisitorAssetInfoDTO infoDTO = new VisitorAssetInfoDTO();
        infoDTO.setTbAsset(tbAsset);
        List<Long> demoVideoIds = assetVideoService.selectVisitorTbAssetVideoForDemoVideo(assetBarCode);
        List<TbDemoVideo> demoVideos = new ArrayList<>();
        if (!demoVideoIds.isEmpty()) {
            for (Long demoVideoId : demoVideoIds) {
                TbDemoVideo demoVideo = demoVideoService.getById(demoVideoId);
                if (demoVideo != null) {
                    demoVideos.add(demoVideo);
                }
            }
        }
        infoDTO.setDemoVideos(demoVideos);

        return AjaxResult.success(infoDTO);
    }

    /**
     * 获取最新研究成果
     */
    @GetMapping("/study/new")
    public AjaxResult getNewStudy(@RequestParam(value = "number", required = false) Long number)
    {
        if (number == null || number <= 0) {
            number = 1L;
        }
        List<SysNotice> sysNoticeList = noticeService.selectNewStudy(number);
        if (sysNoticeList.isEmpty()) {
            return AjaxResult.success("暂无数据", Collections.emptyList());
        }
        if (number == 1) {
            SysNotice sysNotice = sysNoticeList.get(0);
            return AjaxResult.success(sysNotice);
        }
        return AjaxResult.success(sysNoticeList);
    }

    /**
     * 获取通知公告列表
     */
    @GetMapping("/study/list")
    public TableDataInfo studyList(SysNotice notice)
    {
        startPage();
        notice.setNoticeType("3");
        notice.setStatus("0");
        List<SysNotice> list = noticeService.selectNoticeList(notice);
        return getDataTable(list);
    }

    /**
     * 根据通知公告编号获取详细信息
     */
    @GetMapping(value = "/study/detail/{noticeId}")
    public AjaxResult studyDetail(@PathVariable Long noticeId)
    {
        return AjaxResult.success(noticeService.selectNoticeById(noticeId));
    }

    /**
     * 公开演示视频
     *
     * @param tbDemoVideo
     * @return
     */
    @GetMapping("/demo/video")
    public TableDataInfo publicDemoVideo(TbDemoVideo tbDemoVideo) {
        startPage();
        tbDemoVideo.setIsPublic(1);
        List<TbDemoVideo> list = demoVideoService.selectTbDemoVideoList(tbDemoVideo);
        return getDataTable(list);
    }
}
