package com.ruoyi.asset.mapper;

import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.asset.domain.TbAssetBorrowRecord;
import com.ruoyi.asset.domain.dto.TbCountAssetDTO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;

/**
 * 设备借用记录Mapper接口
 * 
 * @author 原动力
 * @date 2023-03-27
 */
@Mapper
public interface TbAssetBorrowRecordMapper extends BaseMapper<TbAssetBorrowRecord>
{
    /**
     * 查询设备借用记录
     * 
     * @param id 设备借用记录主键
     * @return 设备借用记录
     */
    public TbAssetBorrowRecord selectTbAssetBorrowRecordById(Long id);

    /**
     * 查询设备借用记录列表
     * 
     * @param tbAssetBorrowRecord 设备借用记录
     * @return 设备借用记录集合
     */
    public List<TbAssetBorrowRecord> selectTbAssetBorrowRecordList(TbAssetBorrowRecord tbAssetBorrowRecord);

    /**
     * 新增设备借用记录
     * 
     * @param tbAssetBorrowRecord 设备借用记录
     * @return 结果
     */
    public int insertTbAssetBorrowRecord(TbAssetBorrowRecord tbAssetBorrowRecord);

    /**
     * 修改设备借用记录
     * 
     * @param tbAssetBorrowRecord 设备借用记录
     * @return 结果
     */
    public int updateTbAssetBorrowRecord(TbAssetBorrowRecord tbAssetBorrowRecord);

    /**
     * 删除设备借用记录
     * 
     * @param id 设备借用记录主键
     * @return 结果
     */
    public int deleteTbAssetBorrowRecordById(Long id);

    /**
     * 批量删除设备借用记录
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTbAssetBorrowRecordByIds(Long[] ids);

    /**
     * 根据分类统计借用次数
     * @return
     */
    public List<TbCountAssetDTO> countAssetDto();

    /**
     * 统计借出资产设备
     *
     * @return 条数
     */
    @Select("SELECT COUNT(id) from tb_asset_borrow_record where isReturn = 0")
    public int countTbAssetBorrowRecordForNotReturn();
}
