package com.ruoyi.asset.service;

import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.asset.domain.TbAssetVideo;
import com.ruoyi.asset.domain.dto.BatchTbAssetVideoDTO;
import com.ruoyi.asset.domain.dto.TbAssetVideoDTO;

/**
 * 设备视频Service接口
 * 
 * @author 原动力
 * @date 2023-05-06
 */
public interface ITbAssetVideoService extends IService<TbAssetVideo>
{
    /**
     * 查询设备视频
     * 
     * @param id 设备视频主键
     * @return 设备视频
     */
    public TbAssetVideo selectTbAssetVideoById(Long id);

    /**
     * 查询设备视频列表
     * 
     * @param tbAssetVideo 设备视频
     * @return 设备视频集合
     */
    public List<TbAssetVideo> selectTbAssetVideoList(TbAssetVideo tbAssetVideo);

    /**
     * 查询设备视频列表
     *
     * @param tbAssetVideo 设备视频
     * @return 设备视频集合
     */
    public List<TbAssetVideoDTO> selectTbAssetVideoList2(TbAssetVideo tbAssetVideo);

    /**
     * 新增设备视频
     * 
     * @param tbAssetVideo 设备视频
     * @return 结果
     */
    public int insertTbAssetVideo(TbAssetVideo tbAssetVideo);

    /**
     * 批量新增设备视频
     *
     * @param batchTbAssetVideoDTO 多个设备视频
     * @return 结果
     */
    public boolean batchInsertTbAssetVideo(BatchTbAssetVideoDTO batchTbAssetVideoDTO);

    /**
     * 修改设备视频
     * 
     * @param tbAssetVideo 设备视频
     * @return 结果
     */
    public int updateTbAssetVideo(TbAssetVideo tbAssetVideo);

    /**
     * 批量删除设备视频
     * 
     * @param ids 需要删除的设备视频主键集合
     * @return 结果
     */
    public int deleteTbAssetVideoByIds(Long[] ids);

    /**
     * 删除设备视频信息
     * 
     * @param id 设备视频主键
     * @return 结果
     */
    public int deleteTbAssetVideoById(Long id);

    /**
     * 查询公开设备视频绑定资产设备
     * @return 资产设备条形码
     */
    Set<String> selectVisitorTbAssetVideoForAssetBarCode();

    /**
     * 查询公开设备视频绑定视频编码
     *
     * @param assetBarCode 资产设备条形码
     * @return 演示视频编号集合
     */
    List<Long> selectVisitorTbAssetVideoForDemoVideo(String assetBarCode);
}
