package com.ruoyi.asset.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.asset.domain.TbAsset;
import com.ruoyi.asset.domain.TbDemoVideo;
import com.ruoyi.asset.domain.dto.BatchTbAssetVideoDTO;
import com.ruoyi.asset.domain.dto.TbAssetVideoDTO;
import com.ruoyi.asset.mapper.TbAssetMapper;
import com.ruoyi.asset.mapper.TbDemoVideoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.asset.mapper.TbAssetVideoMapper;
import com.ruoyi.asset.domain.TbAssetVideo;
import com.ruoyi.asset.service.ITbAssetVideoService;

import javax.annotation.Resource;

/**
 * 设备视频Service业务层处理
 * 
 * @author 原动力
 * @date 2023-05-06
 */
@Service
public class TbAssetVideoServiceImpl extends ServiceImpl<TbAssetVideoMapper, TbAssetVideo> implements ITbAssetVideoService
{
    @Autowired
    private TbAssetVideoMapper tbAssetVideoMapper;

    @Resource
    private TbAssetMapper tbAssetMapper;

    @Resource
    private TbDemoVideoMapper tbDemoVideoMapper;

    /**
     * 查询设备视频
     * 
     * @param id 设备视频主键
     * @return 设备视频
     */
    @Override
    public TbAssetVideo selectTbAssetVideoById(Long id)
    {
        return tbAssetVideoMapper.selectTbAssetVideoById(id);
    }

    /**
     * 查询设备视频列表
     * 
     * @param tbAssetVideo 设备视频
     * @return 设备视频
     */
    @Override
    public List<TbAssetVideo> selectTbAssetVideoList(TbAssetVideo tbAssetVideo)
    {
        return tbAssetVideoMapper.selectTbAssetVideoList(tbAssetVideo);
    }

    /**
     * 查询设备视频列表
     *
     * @param tbAssetVideo 设备视频
     * @return 设备视频
     */
    @Override
    public List<TbAssetVideoDTO> selectTbAssetVideoList2(TbAssetVideo tbAssetVideo)
    {
        ArrayList<TbAssetVideoDTO> videoDTOS = new ArrayList<>();
        List<TbAssetVideo> tbAssetVideos = tbAssetVideoMapper.selectTbAssetVideoList(tbAssetVideo);
        List<TbAssetVideo> collect = tbAssetVideos.stream().sorted(Comparator.comparing(TbAssetVideo::getAssetBarCode)).collect(Collectors.toList());
        for (TbAssetVideo assetVideo : collect) {
            Long demoVideoId = assetVideo.getDemoVideoId();
            TbDemoVideo tbDemoVideo = tbDemoVideoMapper.selectTbDemoVideoById(demoVideoId);
            String assetBarCode = assetVideo.getAssetBarCode();
            TbAsset tbAsset = tbAssetMapper.selectTbAssetByBarCode(assetBarCode);

            TbAssetVideoDTO tbAssetVideoDTO = BeanUtil.toBean(assetVideo, TbAssetVideoDTO.class);
            tbAssetVideoDTO.setTbDemoVideo(tbDemoVideo);
            tbAssetVideoDTO.setTbAsset(tbAsset);
            videoDTOS.add(tbAssetVideoDTO);
        }

        return videoDTOS;
    }

    /**
     * 新增设备视频
     * 
     * @param tbAssetVideo 设备视频
     * @return 结果
     */
    @Override
    public int insertTbAssetVideo(TbAssetVideo tbAssetVideo)
    {
        return tbAssetVideoMapper.insertTbAssetVideo(tbAssetVideo);
    }

    @Override
    public boolean batchInsertTbAssetVideo(BatchTbAssetVideoDTO batchTbAssetVideoDTO) {
        List<String> assetBarCodes = batchTbAssetVideoDTO.getAssetBarCodes();
        List<Long> videoIds = batchTbAssetVideoDTO.getVideoIds();
        if (assetBarCodes.isEmpty() || videoIds.isEmpty()) {
            return false;
        }
        for (String assetBarCode : assetBarCodes) {
            TbAsset tbAsset = tbAssetMapper.selectTbAssetByBarCode(assetBarCode);
            if (tbAsset == null) {
               continue;
            }
            for (Long videoId : videoIds) {
                TbDemoVideo tbDemoVideo = tbDemoVideoMapper.selectTbDemoVideoById(videoId);
                if (tbDemoVideo == null) {
                    continue;
                }
                Integer count = query().eq("asset_bar_code", assetBarCode).eq("demo_video_id", videoId).count();
                if (count < 1) {
                    tbAssetVideoMapper.insertTbAssetVideo(new TbAssetVideo(assetBarCode, videoId));
                }
            }
        }
        return true;
    }

    /**
     * 修改设备视频
     * 
     * @param tbAssetVideo 设备视频
     * @return 结果
     */
    @Override
    public int updateTbAssetVideo(TbAssetVideo tbAssetVideo)
    {
        return tbAssetVideoMapper.updateTbAssetVideo(tbAssetVideo);
    }

    /**
     * 批量删除设备视频
     * 
     * @param ids 需要删除的设备视频主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetVideoByIds(Long[] ids)
    {
        return tbAssetVideoMapper.deleteTbAssetVideoByIds(ids);
    }

    /**
     * 删除设备视频信息
     * 
     * @param id 设备视频主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetVideoById(Long id)
    {
        return tbAssetVideoMapper.deleteTbAssetVideoById(id);
    }

    @Override
    public Set<String> selectVisitorTbAssetVideoForAssetBarCode() {
        List<TbAssetVideo> list = list();
        // 获取全部资产条形码
        return list.stream()
                .map(TbAssetVideo::getAssetBarCode).collect(Collectors.toCollection(HashSet::new));
    }

    @Override
    public List<Long> selectVisitorTbAssetVideoForDemoVideo(String assetBarCode) {
        List<TbAssetVideo> list = query().eq("asset_bar_code", assetBarCode).list();
        return list.stream().map(TbAssetVideo::getDemoVideoId).collect(Collectors.toList());
    }
}
