package com.ruoyi.web.controller.system;

import java.util.Collections;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.SysNotice;
import com.ruoyi.system.service.ISysNoticeService;

/**
 * 公告 信息操作处理
 * 
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/notice")
public class SysNoticeController extends BaseController
{
    @Autowired
    private ISysNoticeService noticeService;

    /**
     * 获取通知公告列表
     */
    @PreAuthorize("@ss.hasPermi('system:notice:list')")
    @GetMapping("/list")
    public TableDataInfo list(SysNotice notice)
    {
        startPage();
        List<SysNotice> list = noticeService.selectNoticeList(notice);
        return getDataTable(list);
    }

    /**
     * 根据通知公告编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:notice:query')")
    @GetMapping(value = "/{noticeId}")
    public AjaxResult getInfo(@PathVariable Long noticeId)
    {
        return AjaxResult.success(noticeService.selectNoticeById(noticeId));
    }

    /**
     * 新增通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:add')")
    @Log(title = "通知公告", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysNotice notice)
    {
        notice.setCreateBy(getUsername());
        return toAjax(noticeService.insertNotice(notice));
    }

    /**
     * 修改通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:edit')")
    @Log(title = "通知公告", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysNotice notice)
    {
        notice.setUpdateBy(getUsername());
        return toAjax(noticeService.updateNotice(notice));
    }

    /**
     * 删除通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:remove')")
    @Log(title = "通知公告", businessType = BusinessType.DELETE)
    @DeleteMapping("/{noticeIds}")
    public AjaxResult remove(@PathVariable Long[] noticeIds)
    {
        return toAjax(noticeService.deleteNoticeByIds(noticeIds));
    }

    /**
     * 获取最新公告信息
     */
    @PreAuthorize("@ss.hasPermi('system:notice:query')")
    @GetMapping("/newNotice")
    public AjaxResult getNewNotice(@RequestParam(value = "number", required = false) Long number)
    {
        if (number == null || number <= 0) {
            number = 1L;
        }
        List<SysNotice> sysNoticeList = noticeService.selectNewNotice(number);
        if (sysNoticeList.isEmpty()) {
            return AjaxResult.success("暂无公告", Collections.emptyList());
        }
        if (number == 1) {
            SysNotice sysNotice = sysNoticeList.get(0);
            return AjaxResult.success(sysNotice);
        }
        return AjaxResult.success(sysNoticeList);
    }

    /**
     * 获取最新通知信息
     */
    @PreAuthorize("@ss.hasPermi('system:notice:query')")
    @GetMapping("/newInform")
    public AjaxResult getNewInform(@RequestParam(value = "number", required = false) Long number)
    {
        if (number == null || number <= 0) {
            number = 1L;
        }
        List<SysNotice> sysNoticeList = noticeService.selectNewInform(number);
        if (sysNoticeList.isEmpty()) {
            return AjaxResult.success("暂无通知", Collections.emptyList());
        }
        if (number == 1) {
            SysNotice sysNotice = sysNoticeList.get(0);
            return AjaxResult.success(sysNotice);
        }
        return AjaxResult.success(sysNoticeList);
    }
}
